@@ -7,7 +7,7 @@
 # CONDITIONS OF ANY KIND, either express or implied. See the License for the
 # specific language governing permissions and limitations under the License.
 
-import os, random, string, json, datetime, hashlib, hmac, requests, base64
+import os, zlib, random, string, json, datetime, hashlib, hmac, requests, base64
 import time as t
 
 
@@ -454,6 +454,68 @@ class Connect:
         else:
             return interaction
         return results
+    
+    def verify_agent(self, task_name, agent_name):
+        instruct_instance = ''.join(random.choice(string.ascii_letters) for i in range(6))
+        instruct_command = 'get_agents'
+        instruct_args = {'Name': agent_name}
+        agents_list = self.interact_with_task(task_name, instruct_command, instruct_instance, instruct_args)
+        agent_exists = False
+        for agent in agents_list['agents']:
+            if agent_name == agent['name']:
+                return agent
+            else:
+                return False
+
+    def execute_agent_shell_command(self, task_name, agent_name, command):
+        instruct_instance = ''.join(random.choice(string.ascii_letters) for i in range(6))
+        instruct_args = {'Name': agent_name, 'command': command}
+        command_response = self.interact_with_task(task_name, 'agent_shell_command', instruct_instance, instruct_args)
+        if command_response['outcome'] == 'success':
+            command_task_id = command_response['message']['taskID']
+        else:
+            return command_response
+        results = None
+        while not results:
+            instruct_args = {'Name': agent_name}
+            command_results = self.interact_with_task(task_name, 'get_shell_command_results', instruct_instance, instruct_args)
+            if command_results['outcome'] == 'success' and command_results['results']:
+                tmp_results = json.loads(zlib.decompress(base64.b64decode(command_results['results'].encode())).decode())
+                for tmp_result in tmp_results:
+                    if 'taskID' in tmp_result and tmp_result['taskID'] == command_task_id:
+                        if tmp_result['results'] is not None:
+                            results = tmp_result['results']
+            else:
+                results = f'get_shell_command_results for execute_agent_shell_command failed.\n'
+            if not results:
+                t.sleep(10)
+        return results
+    
+    def execute_agent_module(self, task_name, agent_name, module, module_args):
+        instruct_instance = ''.join(random.choice(string.ascii_letters) for i in range(6))
+        instruct_args = {'Agent': agent_name, 'Name': module}
+        for k, v in module_args.items():
+            instruct_args[k] = v
+        module_response = self.interact_with_task(task_name, 'execute_module', instruct_instance, instruct_args)
+        if module_response['outcome'] == 'success':
+            module_task_id = module_response['message']['taskID']
+        else:
+            return module_response
+        results = None
+        while not results:
+            instruct_args = {'Name': agent_name}
+            module_results = self.interact_with_task(task_name, 'get_shell_command_results', instruct_instance, instruct_args)
+            if module_results['outcome'] == 'success' and module_results['results']:
+                tmp_results = json.loads(zlib.decompress(base64.b64decode(module_results['results'].encode())).decode())
+                for tmp_result in tmp_results:
+                    if 'taskID' in tmp_result and tmp_result['taskID'] == module_task_id:
+                        if tmp_result['results'] is not None and 'Job started:' not in tmp_result['results']:
+                            results = tmp_result['results']
+            else:
+                results = f'get_shell_command_results for execute_agent_module failed.\n'
+            if not results:
+                t.sleep(10)
+        return results
 
     def wait_for_c2(self, task_name):
         results = None
